@@ -82,9 +82,8 @@ static void gs_lock_plug_finalize   (GObject         *object);
 
 #define GS_LOCK_PLUG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GS_TYPE_LOCK_PLUG, GSLockPlugPrivate))
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-#define gtk_hbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_HORIZONTAL,Y)
-#define gtk_vbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_VERTICAL,Y)
+#if !GTK_CHECK_VERSION (3, 20, 0)
+#define gtk_widget_set_focus_on_click(a, b) gtk_button_set_focus_on_click(GTK_BUTTON(a), b)
 #endif
 
 struct GSLockPlugPrivate
@@ -1708,7 +1707,7 @@ gs_lock_plug_add_button (GSLockPlug  *plug,
 	g_return_val_if_fail (GS_IS_LOCK_PLUG (plug), NULL);
 	g_return_val_if_fail (button_text != NULL, NULL);
 
-	button = gtk_button_new_from_stock (button_text);
+	button = gtk_button_new_with_label (button_text);
 
 	gtk_widget_set_can_default (button, TRUE);
 
@@ -1773,24 +1772,24 @@ create_page_one_buttons (GSLockPlug *plug)
 	gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (plug->priv->auth_action_area),
 	                                    plug->priv->auth_switch_button,
 	                                    TRUE);
-	gtk_button_set_focus_on_click (GTK_BUTTON (plug->priv->auth_switch_button), FALSE);
+	gtk_widget_set_focus_on_click (GTK_WIDGET (plug->priv->auth_switch_button), FALSE);
 	gtk_widget_set_no_show_all (plug->priv->auth_switch_button, TRUE);
 
 	plug->priv->auth_logout_button =  gs_lock_plug_add_button (GS_LOCK_PLUG (plug),
 	                                  plug->priv->auth_action_area,
 	                                  _("Log _Out"));
-	gtk_button_set_focus_on_click (GTK_BUTTON (plug->priv->auth_logout_button), FALSE);
+	gtk_widget_set_focus_on_click (GTK_WIDGET (plug->priv->auth_logout_button), FALSE);
 	gtk_widget_set_no_show_all (plug->priv->auth_logout_button, TRUE);
 
 	plug->priv->auth_cancel_button =  gs_lock_plug_add_button (GS_LOCK_PLUG (plug),
 	                                  plug->priv->auth_action_area,
 	                                  GTK_STOCK_CANCEL);
-	gtk_button_set_focus_on_click (GTK_BUTTON (plug->priv->auth_cancel_button), FALSE);
+	gtk_widget_set_focus_on_click (GTK_WIDGET (plug->priv->auth_cancel_button), FALSE);
 
 	plug->priv->auth_unlock_button =  gs_lock_plug_add_button (GS_LOCK_PLUG (plug),
 	                                  plug->priv->auth_action_area,
 	                                  _("_Unlock"));
-	gtk_button_set_focus_on_click (GTK_BUTTON (plug->priv->auth_unlock_button), FALSE);
+	gtk_widget_set_focus_on_click (GTK_WIDGET (plug->priv->auth_unlock_button), FALSE);
 
 	gtk_window_set_default (GTK_WINDOW (plug), plug->priv->auth_unlock_button);
 
@@ -1940,21 +1939,37 @@ expand_string_for_label (GtkWidget *label)
 static void
 create_page_one (GSLockPlug *plug)
 {
-	GtkWidget            *align;
 	GtkWidget            *vbox;
 	GtkWidget            *vbox2;
 	GtkWidget            *hbox;
+#if !GTK_CHECK_VERSION(3, 0, 0)
+	GtkWidget            *align;
+#endif
 	char                 *str;
 
 	gs_profile_start ("page one");
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
+	gtk_widget_set_halign (GTK_WIDGET (vbox),
+	                       GTK_ALIGN_CENTER);
+	gtk_widget_set_valign (GTK_WIDGET (vbox),
+	                       GTK_ALIGN_CENTER);
+
+	gtk_notebook_append_page (GTK_NOTEBOOK (plug->priv->notebook), vbox, NULL);
+#else
 	align = gtk_alignment_new (0.5, 0.5, 1, 1);
 	gtk_notebook_append_page (GTK_NOTEBOOK (plug->priv->notebook), align, NULL);
 
 	vbox = gtk_vbox_new (FALSE, 12);
 	gtk_container_add (GTK_CONTAINER (align), vbox);
+#endif
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+	vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 	vbox2 = gtk_vbox_new (FALSE, 0);
+#endif
 	gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
 
 	str = g_strdup ("<span size=\"xx-large\" weight=\"ultrabold\">%s</span>");
@@ -1962,7 +1977,7 @@ create_page_one (GSLockPlug *plug)
 	g_free (str);
 #if GTK_CHECK_VERSION (3, 16, 0)
 	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_time_label), 0.5);
-	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_time_label), 0.5);
+	gtk_label_set_yalign (GTK_LABEL (plug->priv->auth_time_label), 0.5);
 #else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_time_label), 0.5, 0.5);
 #endif
@@ -1974,7 +1989,7 @@ create_page_one (GSLockPlug *plug)
 	g_free (str);
 #if GTK_CHECK_VERSION (3, 16, 0)
 	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_date_label), 0.5);
-	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_date_label), 0.5);
+	gtk_label_set_yalign (GTK_LABEL (plug->priv->auth_date_label), 0.5);
 #else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_date_label), 0.5, 0.5);
 #endif
@@ -1990,7 +2005,11 @@ create_page_one (GSLockPlug *plug)
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_face_image), 0.5, 1.0);
 #endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 	vbox2 = gtk_vbox_new (FALSE, 0);
+#endif
 	gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
 
 	str = g_strdup ("<span size=\"x-large\">%R</span>");
@@ -1999,7 +2018,7 @@ create_page_one (GSLockPlug *plug)
 	expand_string_for_label (plug->priv->auth_realname_label);
 #if GTK_CHECK_VERSION (3, 16, 0)
 	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_realname_label), 0.5);
-	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_realname_label), 0.5);
+	gtk_label_set_yalign (GTK_LABEL (plug->priv->auth_realname_label), 0.5);
 #else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_realname_label), 0.5, 0.5);
 #endif
@@ -2013,22 +2032,31 @@ create_page_one (GSLockPlug *plug)
 	expand_string_for_label (plug->priv->auth_username_label);
 #if GTK_CHECK_VERSION (3, 16, 0)
 	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_realname_label), 0.5);
-	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_realname_label), 0.5);
+	gtk_label_set_yalign (GTK_LABEL (plug->priv->auth_realname_label), 0.5);
 #else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_username_label), 0.5, 0.5);
 #endif
 	gtk_label_set_use_markup (GTK_LABEL (plug->priv->auth_username_label), TRUE);
 	gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_username_label, FALSE, FALSE, 0);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 	vbox2 = gtk_vbox_new (FALSE, 0);
+#endif
 	gtk_box_pack_start (GTK_BOX (vbox), vbox2, TRUE, TRUE, 0);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+#else
 	hbox = gtk_hbox_new (FALSE, 6);
+#endif
 	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);
 
 	plug->priv->auth_prompt_label = gtk_label_new_with_mnemonic (_("_Password:"));
 #if GTK_CHECK_VERSION (3, 16, 0)
 	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_prompt_label), 0.0);
+	gtk_label_set_yalign (GTK_LABEL (plug->priv->auth_prompt_label), 0.5);
 #else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_prompt_label), 0.0, 0.5);
 #endif
@@ -2043,7 +2071,7 @@ create_page_one (GSLockPlug *plug)
 	plug->priv->auth_capslock_label = gtk_label_new ("");
 #if GTK_CHECK_VERSION (3, 16, 0)
 	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_capslock_label), 0.5);
-	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_capslock_label), 0.5);
+	gtk_label_set_yalign (GTK_LABEL (plug->priv->auth_capslock_label), 0.5);
 #else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_capslock_label), 0.5, 0.5);
 #endif
@@ -2123,7 +2151,7 @@ load_theme (GSLockPlug *plug)
 	char       *theme;
 	char       *filename;
 	char       *gtkbuilder;
-	char       *rc;
+	char       *css;
 	GtkBuilder *builder;
 	GtkWidget  *lock_dialog;
 	GError     *error=NULL;
@@ -2144,17 +2172,30 @@ load_theme (GSLockPlug *plug)
 		return FALSE;
 	}
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	filename = g_strdup_printf ("lock-dialog-%s.css", theme);
+	g_free (theme);
+
+	css = g_build_filename (GTKBUILDERDIR, filename, NULL);
+	g_free (filename);
+	if (g_file_test (css, G_FILE_TEST_IS_REGULAR))
+	{
+		GtkCssProvider *style_provider = gtk_css_provider_get_default ();
+		gtk_css_provider_load_from_path (style_provider, css, NULL);
+	}
+	g_free (css);
+#else
 	filename = g_strdup_printf ("lock-dialog-%s.gtkrc", theme);
 	g_free (theme);
 
-	rc = g_build_filename (GTKBUILDERDIR, filename, NULL);
+	css = g_build_filename (GTKBUILDERDIR, filename, NULL);
 	g_free (filename);
-	if (g_file_test (rc, G_FILE_TEST_IS_REGULAR))
+	if (g_file_test (css, G_FILE_TEST_IS_REGULAR))
 	{
-		gtk_rc_parse (rc);
+		gtk_rc_parse (css);
 	}
-	g_free (rc);
-
+	g_free (css);
+#endif
 	builder = gtk_builder_new();
 
 	if (!gtk_builder_add_from_file (builder,gtkbuilder,&error))
@@ -2264,7 +2305,11 @@ gs_lock_plug_init (GSLockPlug *plug)
 	{
 		gs_debug ("Unable to load theme!");
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+		plug->priv->vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
 		plug->priv->vbox = gtk_vbox_new (FALSE, 0);
+#endif
 
 		gtk_container_add (GTK_CONTAINER (plug), plug->priv->vbox);
 
