@@ -62,11 +62,6 @@ enum
 
 #define GS_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GS_TYPE_WINDOW, GSWindowPrivate))
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-#define gtk_hbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_HORIZONTAL,Y)
-#define gtk_vbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_VERTICAL,Y)
-#endif
-
 struct GSWindowPrivate
 {
 	int        monitor;
@@ -212,34 +207,6 @@ gs_window_override_user_time (GSWindow *window)
 	gdk_x11_window_set_user_time (gtk_widget_get_window (GTK_WIDGET (window)), ev_time);
 }
 
-#if !GTK_CHECK_VERSION (3, 0, 0)
-static void
-force_no_pixmap_background (GtkWidget *widget)
-{
-	static gboolean first_time = TRUE;
-
-	if (first_time)
-	{
-		gtk_rc_parse_string ("\n"
-		                     "   style \"gs-theme-engine-style\"\n"
-		                     "   {\n"
-		                     "      bg_pixmap[NORMAL] = \"<none>\"\n"
-		                     "      bg_pixmap[INSENSITIVE] = \"<none>\"\n"
-		                     "      bg_pixmap[ACTIVE] = \"<none>\"\n"
-		                     "      bg_pixmap[PRELIGHT] = \"<none>\"\n"
-		                     "      bg[NORMAL] = { 0.0, 0.0, 0.0 }\n"
-		                     "      bg[INSENSITIVE] = { 0.0, 0.0, 0.0 }\n"
-		                     "      bg[ACTIVE] = { 0.0, 0.0, 0.0 }\n"
-		                     "      bg[PRELIGHT] = { 0.0, 0.0, 0.0 }\n"
-		                     "   }\n"
-		                     "   widget \"gs-window-drawing-area*\" style : highest \"gs-theme-engine-style\"\n"
-		                     "\n");
-		first_time = FALSE;
-	}
-
-	gtk_widget_set_name (widget, "gs-window-drawing-area");
-}
-
 static void
 clear_children (Window window)
 {
@@ -298,7 +265,6 @@ widget_clear_all_children (GtkWidget *widget)
 	gdk_error_trap_pop ();
 #endif
 }
-#endif
 
 #if GTK_CHECK_VERSION (3, 0, 0)
 static void
@@ -311,6 +277,33 @@ gs_window_reset_background_surface (GSWindow *window)
 	cairo_pattern_destroy (pattern);
 	gtk_widget_queue_draw (GTK_WIDGET (window));
 }
+#else
+static void
+force_no_pixmap_background (GtkWidget *widget)
+{
+	static gboolean first_time = TRUE;
+
+	if (first_time)
+	{
+		gtk_rc_parse_string ("\n"
+		                     "   style \"gs-theme-engine-style\"\n"
+		                     "   {\n"
+		                     "      bg_pixmap[NORMAL] = \"<none>\"\n"
+		                     "      bg_pixmap[INSENSITIVE] = \"<none>\"\n"
+		                     "      bg_pixmap[ACTIVE] = \"<none>\"\n"
+		                     "      bg_pixmap[PRELIGHT] = \"<none>\"\n"
+		                     "      bg[NORMAL] = { 0.0, 0.0, 0.0 }\n"
+		                     "      bg[INSENSITIVE] = { 0.0, 0.0, 0.0 }\n"
+		                     "      bg[ACTIVE] = { 0.0, 0.0, 0.0 }\n"
+		                     "      bg[PRELIGHT] = { 0.0, 0.0, 0.0 }\n"
+		                     "   }\n"
+		                     "   widget \"gs-window-drawing-area*\" style : highest \"gs-theme-engine-style\"\n"
+		                     "\n");
+		first_time = FALSE;
+	}
+
+	gtk_widget_set_name (widget, "gs-window-drawing-area");
+}
 #endif
 
 void
@@ -420,7 +413,8 @@ static void
 clear_widget (GtkWidget *widget)
 {
 #if GTK_CHECK_VERSION (3, 0, 0)
-	GdkRGBA      rgba = { 0.0, 0.0, 0.0, 1.0 };
+	GdkRGBA       rgba = { 0.0, 0.0, 0.0, 1.0 };
+	GtkStateFlags state;
 #else
 	GdkColormap *colormap;
 	GdkColor     color = { 0, 0x0000, 0x0000, 0x0000 };
@@ -436,10 +430,12 @@ clear_widget (GtkWidget *widget)
 
 	gs_debug ("Clearing widget");
 
-	gtk_widget_override_background_color (widget, GTK_STATE_FLAG_NORMAL, &rgba);
+	state = gtk_widget_get_state_flags (widget);
+	gtk_widget_override_background_color (widget, state, &rgba);
+	gdk_window_set_background_rgba (gtk_widget_get_window (widget), &rgba);
 	gtk_widget_queue_draw (GTK_WIDGET (widget));
 #else
-	if (! gtk_widget_get_visible (widget))
+	if (!gtk_widget_get_visible (widget))
 	{
 		return;
 	}
@@ -476,12 +472,12 @@ clear_widget (GtkWidget *widget)
 	g_object_unref (style);
 
 	gdk_window_clear (gtk_widget_get_window (widget));
+#endif
 
 	/* If a screensaver theme adds child windows we need to clear them too */
 	widget_clear_all_children (widget);
 
 	gdk_flush ();
-#endif
 }
 
 void
@@ -1106,7 +1102,7 @@ gs_window_real_show (GtkWidget *widget)
 
 static void
 set_info_text_and_icon (GSWindow   *window,
-                        const char *icon_stock_id,
+                        const char *icon_name,
                         const char *primary_text,
                         const char *secondary_text)
 {
@@ -1119,10 +1115,14 @@ set_info_text_and_icon (GSWindow   *window,
 	GtkWidget *primary_label;
 	GtkWidget *secondary_label;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	hbox_content = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
+#else
 	hbox_content = gtk_hbox_new (FALSE, 8);
+#endif
 	gtk_widget_show (hbox_content);
 
-	image = gtk_image_new_from_stock (icon_stock_id, GTK_ICON_SIZE_DIALOG);
+	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_DIALOG);
 	gtk_widget_show (image);
 	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
 #if GTK_CHECK_VERSION (3, 0, 0)
@@ -1131,7 +1131,11 @@ set_info_text_and_icon (GSWindow   *window,
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
 #endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
 	vbox = gtk_vbox_new (FALSE, 6);
+#endif
 	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, FALSE, FALSE, 0);
 
@@ -1699,7 +1703,15 @@ create_lock_socket (GSWindow *window,
                     guint32   id)
 {
 	window->priv->lock_socket = gtk_socket_new ();
+#if GTK_CHECK_VERSION(3, 12, 0)
+	window->priv->lock_box = gtk_grid_new ();
+	gtk_widget_set_halign (GTK_WIDGET (window->priv->lock_box),
+	                       GTK_ALIGN_CENTER);
+	gtk_widget_set_valign (GTK_WIDGET (window->priv->lock_box),
+	                       GTK_ALIGN_CENTER);
+#else
 	window->priv->lock_box = gtk_alignment_new (0.5, 0.5, 0, 0);
+#endif
 	gtk_widget_show (window->priv->lock_box);
 	gtk_box_pack_start (GTK_BOX (window->priv->vbox), window->priv->lock_box, TRUE, TRUE, 0);
 
@@ -1763,8 +1775,7 @@ static void
 shake_dialog (GSWindow *window)
 {
 	int   i;
-	guint left;
-	guint right;
+	guint start, end;
 
 	window->priv->dialog_shake_in_progress = TRUE;
 
@@ -1772,13 +1783,13 @@ shake_dialog (GSWindow *window)
 	{
 		if (i % 2 == 0)
 		{
-			left = 30;
-			right = 0;
+			start = 30;
+			end = 0;
 		}
 		else
 		{
-			left = 0;
-			right = 30;
+			start = 0;
+			end = 30;
 		}
 
 		if (! window->priv->lock_box)
@@ -1786,10 +1797,16 @@ shake_dialog (GSWindow *window)
 			break;
 		}
 
+#if GTK_CHECK_VERSION(3, 12, 0)
+		gtk_widget_set_margin_start (GTK_WIDGET (window->priv->lock_box),
+		                             start);
+		gtk_widget_set_margin_end (GTK_WIDGET (window->priv->lock_box),
+		                           end);
+#else
 		gtk_alignment_set_padding (GTK_ALIGNMENT (window->priv->lock_box),
 		                           0, 0,
-		                           left,
-		                           right);
+		                           start, end);
+#endif
 
 		while (gtk_events_pending ())
 		{
@@ -2740,13 +2757,20 @@ create_info_bar (GSWindow *window)
 	gtk_box_pack_end (GTK_BOX (window->priv->vbox), window->priv->info_bar, FALSE, FALSE, 0);
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void
+on_drawing_area_realized (GtkWidget *drawing_area)
+{
+	GdkRGBA black = { 0.0, 0.0, 0.0, 1.0 };
+
+	gdk_window_set_background_rgba (gtk_widget_get_window (drawing_area),
+	                                &black);
+}
+#endif
+
 static void
 gs_window_init (GSWindow *window)
 {
-#if GTK_CHECK_VERSION (3, 0, 0)
-	GdkRGBA black = { 0.0, 0.0, 0.0, 1.0 };
-
-#endif
 	window->priv = GS_WINDOW_GET_PRIVATE (window);
 
 	window->priv->geometry.x      = -1;
@@ -2778,7 +2802,11 @@ gs_window_init (GSWindow *window)
 	                       | GDK_ENTER_NOTIFY_MASK
 	                       | GDK_LEAVE_NOTIFY_MASK);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	window->priv->vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
+#else
 	window->priv->vbox = gtk_vbox_new (FALSE, 12);
+#endif
 	gtk_widget_show (window->priv->vbox);
 	gtk_container_add (GTK_CONTAINER (window), window->priv->vbox);
 
@@ -2789,9 +2817,10 @@ gs_window_init (GSWindow *window)
 #endif
 	gtk_box_pack_start (GTK_BOX (window->priv->vbox), window->priv->drawing_area, TRUE, TRUE, 0);
 #if GTK_CHECK_VERSION (3, 0, 0)
-        gtk_widget_realize (window->priv->drawing_area);
-        gdk_window_set_background_rgba (gtk_widget_get_window (window->priv->drawing_area), &black);
-
+	g_signal_connect (window->priv->drawing_area,
+	                  "realize",
+	                  G_CALLBACK (on_drawing_area_realized),
+	                  NULL);
 #endif
 	create_info_bar (window);
 
